##  file.tcl
##
##		This file implements code for
##		files i/o
##
##  Copyright (c) 1998 <Irrational Numbers> : <buraldo@maganet.net> 
##
##  This program is free software; you can redistibute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation.
##
##  See the file "license" for information on usage and
##  redistribution of this program, and for a disclaimer of all warranties.
##
##  VER 1.1


namespace eval FILE {

set diagramName untitled
set diagramDir  ""

proc newDiagram {w} {
	variable diagramName

	cleanDiagram $w
	CVS::draw_paper $w
	set diagramName untitled
	CVS::set_DocTitle $w $diagramName
	CVS::node $w startSymb [expr ($CVS::XMIN + $CVS::XMAX)/2] 30

	wm title . " HMSC: $diagramName"
}

proc cleanDiagram {w} {
	$w delete all
        MUI::create_select_box $w

	foreach v [info var "::CVS::grp#*"] {
		catch {unset $v}  ; ## it could be unset, yet
	}
}

proc exiting {w} {
	while { 1 } {
		set answer [ tk_messageBox -icon warning  -title "Exiting..." \
			-message "You are going to exit\n\nSave the diagram ?" \
			-type yesnocancel ]
		if { $answer == "yes" } {
			if { [saveDiagram $w] } { 
				exit 
			}
		} elseif { $answer == "no"} { 
			exit 
		} else {
			break
		}
	}
}

proc saveDiagram {w} {
	set filename [tk_getSaveFile -defaultextension .pad  \
		-filetypes { { {PAD Files} {.pad} }} \
	]
	if { $filename != "" } {
		variable diagramName
		variable diagramDir 
		set diagramName $filename
		set FILE::ui_msgline "Saving diagram ..."
		save000 $w $diagramName
		set diagramDir [file dirname $diagramName]
		cd $diagramDir
		wm title . " HMSC: $diagramName"
		set FILE::ui_msgline ""
		return 1
	}
	return 0
}

proc save000 { w filename } {
	global HMSC_VER

	set fid [open $filename w]
	puts $fid "# HMSC version $HMSC_VER"
	puts $fid "proc HMSCload {w} {"

	puts $fid "CVS::set_paper \$w [CVS::get_paper $w]"
	set txt [CVS::get_DocTitle $w]
	puts $fid "CVS::set_DocTitle \$w \{$txt\}"

	foreach id [$w find withtag node] {
		set nodeGrp [lgrep [$w gettags $id] "grp#*"]
		set params [CVS::get_node $w $nodeGrp]
		puts $fid "set map($nodeGrp) \[CVS::node \$w $params \]"
	}
	foreach id [$w find withtag arc] {
		set arcGrp [lgrep [$w gettags $id] "grp#*"]
		set params [CVS::get_arc $w $arcGrp]
		set n0 [lindex $params 0]
		set n1 [lindex $params 1]
		set otherParams [lrange $params 2 end]
		set n0 "\$map($n0)"
		set n1 "\$map($n1)"
		puts $fid "CVS::arc \$w $n0 $n1 $otherParams"
	}
	puts $fid "}"
	close $fid
}

proc openDiagram {w} {
	set filename [tk_getOpenFile \
		-filetypes { { {PAD Files} {.pad} }} \
	]
	if { $filename != "" } {		
		variable diagramName
		variable diagramDir
		set diagramName $filename
		set FILE::ui_msgline "Loading diagram ..."
		cleanDiagram $w
		open000 $w $diagramName
		set diagramDir [file dirname $diagramName]
		cd $diagramDir
		wm title . " HMSC: $diagramName"
		set FILE::ui_msgline ""
	}
}

proc open000 {w filename} {
	source $filename
	HMSCload $w
}


proc printToFile {w} {
	set filename [tk_getSaveFile -defaultextension .ps  \
		-title "Print PostScript to file ..." \
		-filetypes { { {ps PostScript} {.ps} }} \
	]
	if { $filename != "" } {
		set FILE::ui_msgline "Printing diagram ..."
		$w postscript -file $filename -colormode mono \
			-x $CVS::XMIN  -y $CVS::YMIN \
			-width [expr $CVS::XMAX - $CVS::XMIN] \
			-height [expr $CVS::YMAX - $CVS::YMIN]
		set FILE::ui_msgline ""
		return 1
	}
	return 0
}


proc saveZ120 { cvs } {
	set filename [tk_getSaveFile -defaultextension hms  \
		-filetypes { { {HMSC Files} {.hms} }} \
	]
	if { $filename == "" } return

	set fID [open $filename w]
	set errCode [CVS::writeZ120 $fID $cvs]
	close $fID
	if {$errCode} {
		file delete $filename
	}
}

} ; ## = end of namespace =====================================================
